import subprocess as sp
from . import General as Gen

rar_exe_path = f'{Gen.file_path(__file__)}/bin/Rar.exe'


# dd

def list_content(rar_path: str):
    rar = sp.Popen([rar_exe_path, '-ep', 'lb', rar_path],
                   stdin=sp.PIPE,
                   stdout=sp.PIPE,
                   stderr=sp.PIPE)
    result = rar.communicate()[0].decode()
    return result.splitlines()


def add_to(rar_path: str, in_file: str, data: bytes):
    rar = sp.Popen([rar_exe_path, '-rr3', '-m0', f'-si{in_file}', 'a', rar_path],
                   stdin=sp.PIPE,
                   stdout=sp.PIPE,
                   stderr=sp.PIPE)
    result = rar.communicate(input=data)[0].decode()
    return result



import subprocess as sp
import time

import aria2p

from . import OS
from . import General
from . import Logger as Log


class Aria2:
    def __init__(self, host='0.0.0.0', port=6809, secret='1234', timeout=15):
        # %% create server
        a2c_bin = General.gen_os_exe_name('aria2c')
        if OS.is_windows():
            a2c_bin = f'{General.file_path(__file__)}/bin/{a2c_bin}'
            Log.info(f'{a2c_bin=}')
        self.server = sp.Popen(
            [a2c_bin, '--enable-rpc', f'--rpc-listen-port={port}',
             f'--rpc-secret={secret}'],
            stdin=sp.PIPE,
            stdout=sp.PIPE,
            stderr=sp.PIPE)
        Log.info(f'server started: {host=}, {port=}, secret=*****, {timeout=}...')
        # %%

        # %% wait for server to init
        r = ''
        while True:
            time.sleep(1)
            r += self.server.stdout.read(80).decode().lower()
            print(r)
            if r.find(f'listening on tcp port {port}') > -1:
                Log.info('server initialization completed...')
                break
            Log.info('waiting for server init...')
        # %%

        # %% then create client connection to rpc
        self.client = aria2p.API(
            aria2p.Client(host=f'http://127.0.0.1', port=port, secret=secret, timeout=timeout))
        self.client.set_global_options({'split': 8})

        Log.info(f'aria2p API client created: http://127.0.0.1:{port}, secret=*****, {timeout=}...')
        # %%

        self.host = host
        self.port = port
        self.secret = secret
        self.timeout = timeout

    def __del__(self):
        if self.server is not None:
            self.server.terminate()
            Log.info('server terminated.')

    def add_uris(self, uris: [str, list], _dir: str = None) -> aria2p.Download:
        if type(uris) is str:
            uris = [uris]

        if _dir is not None:
            _dir = {'dir': _dir}

        return self.client.add_uris(uris=uris, options=_dir)

    def download_file_sync(self, uris: [str, list], _dir: str = None):
        download = self.add_uris(uris, _dir)
        while not download.is_complete:
            time.sleep(.5)
            download.update()
            Log.info(f'{download.connections=}')
            Log.info(f'downloading: {download.progress_string()}')
        return download


'''aria2 = Aria2()
d = aria2.download_file_sync(uris='https://dl2.soft98.ir/soft/o/Opera.81.0.4196.60.x64.rar', _dir='d:/')

print(len(d.files))
for file in d.files:
    print(file.path)
'''



import logging
import logging.handlers as lh
import os
from datetime import datetime as dt


def make_dirs_if_not_exists(path):
    if not os.path.exists(path):
        os.makedirs(path)


def now_dt_str():
    return dt.now().strftime('%Y-%m-%d.%H-%M-%S')


working_dir = os.getcwd()
logs_dir = f'{working_dir}/logs'

make_dirs_if_not_exists(logs_dir)

# PYTHONUTF8=1
os.environ['PYTHONUTF8'] = '1'

# Logger

# Formatter
formatter = logging.Formatter(u"%(asctime)s:%(levelname)s:%(message)s")

# File Handler
# file_handler=logging.FileHandler(f'{working_dir}/logs/{now_dt_str()}.log', encoding='utf8')

# Timed Rotating File Handler
timed_rotating_handler = lh.TimedRotatingFileHandler(f'{logs_dir}/{now_dt_str()}.log', when='midnight',
                                                     encoding='utf8')
timed_rotating_handler.suffix = '~%Y-%m-%d.%H-%M-%S.log'
timed_rotating_handler.setFormatter(formatter)

# Stream Handler
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)

# Logger Config
logger = logging.getLogger()
logger.setLevel(logging.INFO)
logger.addHandler(timed_rotating_handler)
logger.addHandler(stream_handler)


def debug(text):
    try:
        logging.debug(text)
    except Exception as ex:
        print(ex)


def info(text):
    try:
        logging.info(text)
    except Exception as ex:
        print(ex)


def error(text):
    try:
        logging.error(text)
    except Exception as ex:
        print(ex)



import requests as req
import time
from . import Logger as Log


class ResponseBodyEmpty(Exception):
    def __init__(self):
        super().__init__('response body empty')


def print_exception(message, retry):
    Log.info(f'retry: {retry} ->')
    Log.info(message)


def get_url(url, retry_sleep=.5, retry=15, timeout=30, verify=False, stream=False):
    for i in range(retry):
        response = None
        try:
            response = req.get(url, timeout=timeout, verify=verify, stream=stream)
            if response is None:
                continue
            if response.status_code != 200:
                raise Exception(f'status code: {response.status_code}')
            if 'Content-Length' in response.headers \
                    and int(response.headers['Content-Length']) <= 0:
                raise ResponseBodyEmpty()
            return response
        except ResponseBodyEmpty as ex:
            print_exception(f'{ex}, {url}', i + 1)
            if i >= 1:
                break
        except Exception as ex:
            msg = f'ex: {ex.__str__()}'
            if response is not None:
                msg += f', {url}, {response.status_code}, {response.headers=}'
            print_exception(msg, i + 1)
        time.sleep(retry_sleep)
    return None


def get_url_raw(**kwargs):
    response = get_url(stream=True, **kwargs)
    return b'' if response is None else response.raw.data


def get_url_utf8(**kwargs):
    response = get_url(**kwargs)
    return '' if response is None else response.content.decode('utf8')


def write_bytes_to_file(path, data):
    with open(path, 'wb') as f:
        f.write(data)


def download_to_file(url: str, path: str, raw=False, _return=False) -> bytes:
    response = get_url(url=url, stream=raw)

    if not response:
        raise Exception('invalid response from server received.')

    if raw:
        data = response.raw.data
    else:
        data = response.content

    write_bytes_to_file(path, data)

    if _return:
        return data



import time
from datetime import datetime as dt
import gzip
import os
from datetime import datetime, timedelta
from random import randint
from pathlib import Path
from . import OS


def now_dt_str():
    return dt.now().strftime('%Y-%m-%d.%H-%M-%S')


def decompress_gzip(data):
    return gzip.decompress(data)


def compress_gzip(data):
    return gzip.compress(data)


def compress_str_gzip(text: str):
    return compress_gzip(text.encode())


def make_dirs_if_not_exists(path):
    if not os.path.exists(path):
        os.makedirs(path)


def file_exists(path: str):
    return os.path.isfile(path)


def dir_exists(path: str):
    return os.path.isdir(path)


def dt_now():
    return datetime.now()


def time_delta_seconds(d1: datetime, d2: datetime):
    return (d1 - d2).total_seconds()


def sec_until_time(_t: time):
    start = datetime.combine(datetime.today(), _t)
    sec_until_datetime(start)


def sec_until_datetime(_dt: datetime):
    next_run = time_delta_seconds(_dt, dt_now())
    if next_run <= 0:
        _dt += timedelta(days=1)
        next_run = time_delta_seconds(_dt, dt_now())
    return next_run


def sec_until_midnight():
    midnight = datetime.today() + timedelta(days=1)
    midnight = (midnight - dt_now()).total_seconds()
    return midnight


def sleep_rand_time(a, b):
    time.sleep(randint(a, b))


def is_hour_between(h1, h2):
    _now = dt_now()
    if _now.hour >= h1 or _now.hour <= h2:
        return True
    return False


def file_modification_time(path):
    try:
        mtime = os.path.getmtime(path)
        mtime = dt.fromtimestamp(mtime)
    except:
        return dt(1, 1, 1)
    return mtime


def file_path(_file_):
    return os.path.dirname(_file_)


def recursive_list_path(path, dirs=False, files=True):
    items = []

    for p in Path(path).rglob('*'):
        if (not files and p.is_file()) or (not dirs and p.is_dir()):
            continue

        items.append(p.as_posix())

    return items


def read_file_degzip_utf8(file) -> str:
    file = Path(file)
    file = file.read_bytes()
    file = decompress_gzip(file).decode('utf8')
    return file


def gen_os_exe_name(file: str):
    if OS.is_windows():
        if not file.endswith('.exe'):
            file = f'{file}.exe'
    else:
        if file.endswith('.exe'):
            file.removesuffix('.exe')
    return file



from setuptools import setup

setup(
    name='RevUtils',
    packages=['RevUtils'],
    version='2021.12.5.18.9.50.765532',
    license='MIT',
    description='Utils Created And Used By RevEngine3r',
    author='Saeed',
    author_email='RevEngine3r@Gmail.Com',
    url='https://github.com/RevEngine3r/RevUtilities',
    download_url='https://github.com/RevEngine3r/RevUtilities/archive/refs/heads/master.zip',
    keywords=['RAR', 'RevEngine3r', 'Persian', 'Tools'],
    install_requires=[
        'requests',
        'beautifulsoup4',
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    package_data={'RevUtils': ['*.exe']}
)



from . import IO
from . import General

from datetime import datetime
import bs4

holiday_last_check_datetime = datetime(1, 1, 1)
holiday_last_result = False

finglish_tr = str.maketrans({
    'آ': 'a',
    'ا': 'a',
    'ب': 'b',
    'پ': 'p',
    'ت': 't',
    'ث': 's',
    'ج': 'j',
    'چ': 'ch',
    'ح': 'h',
    'خ': 'kh',
    'د': 'd',
    'ذ': 'z',
    'ر': 'r',
    'ز': 'z',
    'ژ': 'zh',
    'س': 's',
    'ش': 'sh',
    'ص': 's',
    'ض': 'z',
    'ط': 't',
    'ظ': 'z',
    'ع': 'i',
    'غ': 'gh',
    'ف': 'f',
    'ق': 'g',
    'ک': 'k',
    'گ': 'g',
    'ل': 'l',
    'م': 'm',
    'ن': 'n',
    'و': 'v',
    'ه': 'h',
    'ی': 'y',
})


def is_today_holiday():
    global holiday_last_result, holiday_last_check_datetime
    if General.dt_now().day != holiday_last_check_datetime.day:
        data = IO.get_url_utf8(url='https://www.time.ir')
        bs = bs4.BeautifulSoup(markup=data, features="html.parser")
        data = bs.find(name='div', attrs={'class': 'today'})
        data = data.find(name='div', attrs={'class': 'holiday'})
        holiday_last_result = False if data is None else True
        holiday_last_check_datetime = General.dt_now()
    return holiday_last_result


def fa_to_finglish(text: str) -> str:
    return text.translate(finglish_tr)



import threading
import time
from . import Logger

max_threads = 0
current_threads = 0


def target_wrapper(target, args=()):
    global current_threads, max_threads

    Logger.info(f'stats: {current_threads}/{max_threads} threads...')
    try:
        current_threads += 1
        Logger.info(f'Starting Thread #{current_threads}...')
        target(*args)
    except Exception as ex:
        Logger.info(f'Thread #{current_threads}: {ex}')
    finally:
        Logger.info(f'Thread #{current_threads} Finished...')
        current_threads -= 1


def start_thread(target, args=(), _daemon=False):
    global current_threads, max_threads

    t = threading.Thread(target=target_wrapper, args=(target, args), daemon=_daemon)

    if 0 < max_threads <= current_threads:
        Logger.info('Max Threads Limit Exceed, Waiting...')
        while 0 < max_threads <= current_threads:
            time.sleep(.1)

    Logger.info('Starting Thread...')
    t.start()
    return t


def start_daemon(target, args=()):
    Logger.info('Starting Daemon...')
    return start_thread(target=target, args=args, _daemon=True)


def target_persistent(target, args=()):
    Logger.info('Starting Persistent Thread...')
    while True:
        try:
            target(*args)
        except Exception as ex:
            Logger.info(f'Persistent Thread Exception: {ex}')
        finally:
            Logger.info('Restarting Persistent Thread...')


def start_persistent_daemon(target, args=()):
    Logger.info('Starting Persistent Daemon...')
    return start_daemon(target=target_persistent, args=(target, args))


def is_all_threads_done():
    return current_threads <= 0


def join():
    while not is_all_threads_done():
        time.sleep(.1)
